@@ -41,6 +41,18 @@ from miss_hit_core import s_parser
 from miss_hit_core import s_ast
 
 from miss_hit_core.version import GITHUB_ISSUES, VERSION, FULL_NAME
+from miss_hit_core.m_language import (Base_MATLAB_Language,
+                                      Base_Octave_Language)
+
+
+def has_relevant_extension(path, process_slx=True):
+    assert isinstance(path, str)
+    assert isinstance(process_slx, bool)
+
+    if process_slx:
+        return os.path.splitext(path)[1] in (".m", ".tst", ".slx")
+    else:
+        return os.path.splitext(path)[1] in (".m", ".tst")
 
 
 def create_basic_clp(epilog=None):
@@ -54,7 +66,26 @@ def create_basic_clp(epilog=None):
     ap.add_argument("-v", "--version",
                     action="store_true",
                     default=False,
-                    help="Show version and exit")
+                    help="Show MISS_HIT version and exit")
+
+    language_options = ap.add_mutually_exclusive_group()
+    language_options.add_argument("--octave",
+                                  default=None,
+                                  metavar="VERSION",
+                                  help=("Enable support for the Octave"
+                                        " language at a given version,"
+                                        " e.g. 4.4. If no version is specified"
+                                        " then latest supported is used."))
+
+    language_options.add_argument("--matlab",
+                                  default=None,
+                                  metavar="VERSION",
+                                  help=("Enable support for the MATLAB"
+                                        " language at a given version,"
+                                        " e.g. 2021a. If no version is "
+                                        " specified then the latest supported"
+                                        " is used."))
+
     ap.add_argument("--include-version",
                     action="store_true",
                     default=False,
@@ -109,17 +140,6 @@ def create_basic_clp(epilog=None):
                                 default=False,
                                 help="Don't show line-context on messages")
 
-    language_options = ap.add_argument_group("language options")
-    rv["language_options"] = language_options
-
-    language_options.add_argument("--octave",
-                                  default=False,
-                                  action="store_true",
-                                  help=("Enable support for the Octave"
-                                        " language. Note: This is highly"
-                                        " incomplete right now, only the"
-                                        " # comments are supported."))
-
     debug_options = ap.add_argument_group("debugging options")
     rv["debug_options"] = debug_options
 
@@ -161,6 +181,22 @@ def parse_args(clp):
     except LookupError:
         clp["ap"].error("invalid encoding '%s'" % options.input_encoding)
 
+    options.language = None
+    if options.octave is not None:
+        try:
+            major, minor = Base_Octave_Language.parse_version(options.octave)
+            options.language = Base_Octave_Language.get_version(major, minor)
+        except ValueError as verr:
+            clp["ap"].error(verr.args[0])
+        del options.octave
+    if options.matlab is not None:
+        try:
+            major, minor = Base_MATLAB_Language.parse_version(options.matlab)
+            options.language = Base_MATLAB_Language.get_version(major, minor)
+        except ValueError as verr:
+            clp["ap"].error(verr.args[0])
+        del options.matlab
+
     return options
 
 
@@ -268,7 +304,7 @@ def execute(mh, options, extra_options, back_end,
                 for path, dirs, files in os.walk(path_root):
                     container.add(os.path.normpath(path))
                     for f in files:
-                        if f.endswith(".m") or f.endswith(".slx"):
+                        if has_relevant_extension(f):
                             container.add(
                                 os.path.normpath(os.path.join(path, f)))
                     irrelevant_dirs = set(d for d in dirs
@@ -296,10 +332,10 @@ def execute(mh, options, extra_options, back_end,
                 # path.
                 item_list = [(False, item)
                              for item in sorted(code_in_path)
-                             if item.endswith(".m") or item.endswith(".slx")]
+                             if has_relevant_extension(item)]
                 item_list += [(True, item)
                               for item in sorted(test_in_path)
-                              if item.endswith(".m") or item.endswith(".slx")]
+                              if has_relevant_extension(item)]
 
             # Post-process to use relative directories
             item_list = [(in_test_dir, os.path.relpath(item))
@@ -349,8 +385,7 @@ def execute(mh, options, extra_options, back_end,
                     if not os.path.isfile(os.path.join(path, f)):
                         # This removes broken symlinks
                         continue
-                    if f.endswith(".m") or (f.endswith(".slx") and
-                                            process_slx):
+                    if has_relevant_extension(f, process_slx):
                         work_list.append(
                             work_package.create(in_test_dir,
                                                 os.path.join(path, f),
@@ -358,7 +393,7 @@ def execute(mh, options, extra_options, back_end,
                                                 mh,
                                                 options, extra_options))
 
-        elif item.endswith(".m") or (item.endswith(".slx") and process_slx):
+        elif has_relevant_extension(item, process_slx):
             work_list.append(work_package.create(in_test_dir,
                                                  item,
                                                  options.input_encoding,
